@@ -228,7 +228,7 @@ class XMLiElement(object):
             value = "0" if not value else str(value)
 
         tag = root.ownerDocument.createElement(name)
-        value = to_byte_string(value)
+        value = value.decode('utf-8')
         if cdata:
             tag.appendChild(root.ownerDocument.createCDATASection(value))
         else:
@@ -897,14 +897,18 @@ class Invoice(ExtensibleXMLiElement):
         try:
             from Crypto.PublicKey import RSA
         except ImportError:
-            raise ImportError('PyCrypto 2.5 module is required to enable ' \
-                              'XMLi signing. Please visit:\n' \
-                              'http://pycrypto.sourceforge.net/')
-
+            raise ImportError('PyCrypto 2.5 or more recent module is ' \
+                              'required to enable XMLi signing.\n' \
+                              'Please visit: http://pycrypto.sourceforge.net/')
+            
+        if not isinstance(private, RSA._RSAobj):
+            private = RSA.importKey(private.read(), passphrase=passphrase)
+        
+        if not isinstance(public, RSA._RSAobj):
+            public = RSA.importKey(public.read())
+            
         return to_unicode(xmldsig.sign(to_byte_string(self.to_string()),
-                                       RSA.importKey(private.read(),
-                                                     passphrase=passphrase),
-                                       RSA.importKey(public.read())))
+                                       private, public))
 
 
 class DeliveryMethod(ExtensibleXMLiElement):
@@ -967,7 +971,7 @@ class DeliveryMethod(ExtensibleXMLiElement):
         for n, v in { "method": self.method, "status": self.status,
                      "date":self.date}.items():
             if is_empty_or_none(v):
-                raise DeliveryMethodException("'%s' attribute cannot be " \
+                raise DeliveryMethodError("'%s' attribute cannot be " \
                                        "empty or None." % n)
 
         doc = Document()
@@ -1269,7 +1273,7 @@ class Line(ExtensibleXMLiElement):
         @param precision: int number of decimal places
         @return: Decimal
         '''
-        return quantize(self.unit_price + self.quantity, precision)
+        return quantize(self.unit_price * self.quantity, precision)
 
     @property
     def total_discounts(self):
@@ -1503,7 +1507,7 @@ class Treatment(XMLiElement):
                      "name": self.name,
                      "description": self.description}.items():
             if is_empty_or_none(v):
-                raise TreatmentException("'%s' attribute cannot be empty " \
+                raise TreatmentError("'%s' attribute cannot be empty " \
                                          "or None." % n)
 
         doc = Document()
